package de.intevation.flys.client.server;

import org.w3c.dom.Document;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import de.intevation.artifacts.common.utils.ClientProtocolUtils;
import de.intevation.artifacts.common.utils.CreationFilter;

import de.intevation.artifacts.httpclient.exceptions.ConnectionException;
import de.intevation.artifacts.httpclient.http.HttpClient;
import de.intevation.artifacts.httpclient.http.HttpClientImpl;

import de.intevation.flys.client.shared.exceptions.ServerException;
import de.intevation.flys.client.shared.model.Artifact;

import de.intevation.flys.client.shared.model.Recommendation;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class ArtifactHelper {

    /** The error message key that is thrown if an error occured while artifact
     * creation.*/
    public static final String ERROR_CREATE_ARTIFACT = "error_create_artifact";


    private ArtifactHelper() {
    }


    /**
     * @param factory ArtifactFactory to use.
     */
    public static Artifact createArtifact(
        String         serverUrl,
        String         locale,
        String         factory,
        Recommendation recommendation)
    throws ServerException
    {
        System.out.println("ArtifactHelper.create");

        String         uuid;
        String         ids;
        CreationFilter filter;

        if (recommendation != null) {
            uuid   = recommendation.getMasterArtifact();
            ids    = recommendation.getIDs();
            filter = convertFilter(recommendation.getFilter());
        }
        else {
            uuid   = null;
            ids    = null;
            filter = null;
        }

        Document create = ClientProtocolUtils.newCreateDocument(
            factory, uuid, ids, filter);

        HttpClient client = new HttpClientImpl(serverUrl, locale);

        try {
            return (Artifact) client.create(create, new FLYSArtifactCreator());
        }
        catch (ConnectionException ce) {
            System.err.println(ce.getLocalizedMessage());
        }

        throw new ServerException(ERROR_CREATE_ARTIFACT);
    }


    /**
     * Create CreationFilter from Recommendation.Filter.
     */
    public static CreationFilter convertFilter(Recommendation.Filter filter) {

        if (filter == null) {
            return null;
        }

        CreationFilter cf = new CreationFilter();

        Map<String, List<Recommendation.Facet>> outs = filter.getOuts();

        for (Map.Entry<String, List<Recommendation.Facet>> entry:
            outs.entrySet()) {
            List<Recommendation.Facet> rfs = entry.getValue();
            List<CreationFilter.Facet> cfs =
                new ArrayList<CreationFilter.Facet>(rfs.size());
            for (Recommendation.Facet rf: rfs) {
                cfs.add(new CreationFilter.Facet(rf.getName(), rf.getIndex()));
            }
            cf.add(entry.getKey(), cfs);
        }

        return cf;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
